/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _GB2_CMD_LINE_REGISTRY_
#define _GB2_CMD_LINE_REGISTRY_

#include "core_api/core_api.h"

#include <QtCore/QStringList>
#include <QtCore/QMap>

namespace GB2 {

class CMDLineHelpProvider;

class GB2_COREAPI_EXPORT CMDLineRegistry {
public:
    /* Reads command arguments into UGENE's command line parameters 
       
       Convention:
       --paramName=paramValue -> (paramName, paramValue)
       -paramName paramValue -> (paramName, paramValue)
       paramName1 paramName2  -> (paramName, ""), (paramName2, "")
       
       Special cases like --paramName=<EOL or another paramName> or -p <EOL or another paramName> 
       will lead to empty values for the given params
    */
    CMDLineRegistry(const QStringList& arguments);
    virtual ~CMDLineRegistry(){};

    const QStringList& getOrderedParamNames() const {return orderedParamNames;}
    const QMap<QString, QString>& getParams() const {return params;}

    void registerCMDLineHelpProvider(CMDLineHelpProvider* provider);
    void unregisterCMDLineHelpProvider(CMDLineHelpProvider* provider);

    void dumpHelp() const;

private:
    QStringList             orderedParamNames;
    QMap<QString, QString>  params;

    QList<CMDLineHelpProvider* > helpProviders;
};

}//namespace

#endif
