/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _GB2_GRAPHICS_BRANCH_ITEM_H_
#define _GB2_GRAPHICS_BRANCH_ITEM_H_

#include <QtGui/QAbstractGraphicsShapeItem>
#include <QtGui/QFont>
#include <QtCore/QMap>
#include <QtCore/QList>
#include <core_api/Task.h>

namespace GB2 {

class PhyNode;
class GraphicsButtonItem;

class GraphicsBranchItem: public QAbstractGraphicsShapeItem {
    friend class BranchCreationTask;
public:
    enum Direction { up, down };
    static const qreal DEFAULT_WIDTH;
    static const qreal MAXIMUM_WIDTH;
    static const int TEXT_SPACE;
    static const int SELECTED_PEN_WIDTH;

private:
    GraphicsBranchItem(const QFont &font, const QString& name, GraphicsBranchItem* pitem);
    GraphicsBranchItem(const QFont &font, qreal d);
    GraphicsBranchItem(const QFont &font, qreal x, qreal y, const QString& name, qreal d);
    GraphicsBranchItem(const QFont &font, qreal x, qreal y, const QString& name);

    void setYW(int new_y);
    void initPos();
    void initText(const QFont &font, qreal d);

    qreal width, height;
    Direction direction;
    bool visible, collapsed;

public:
    GraphicsBranchItem();

    QRectF boundingRect() const;
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
    QPainterPath shape() const;
    void setParentItem(QGraphicsItem *item);

    qreal getWidth() const { return width; }
    qreal getHeight() const { return height; }
    void setWidthW(qreal w) { width = w; }
    void setHeightW(qreal h) { height = h; }
    void setWidth(qreal w);
    void setHeight(qreal h);
    void setDirection(Direction d);

    GraphicsButtonItem *button;
    QGraphicsSimpleTextItem *distanceText;
    QGraphicsSimpleTextItem *nameText;
    void setVisibleW(bool v) { visible = v; }

    void setSelected(bool sel);
    void collapse();
    void zoom();
};

class BranchCreationTask: public Task {
    Q_OBJECT

    QMap<int, GraphicsBranchItem*> vers;
    PhyNode *node;
    QFont font;
    QMap<PhyNode*, int> aligns;
    int size, height;
    qreal minDistance, maxDistance;

    void initPos(GraphicsBranchItem *item);
    GraphicsBranchItem* getNode(PhyNode *node);

signals:
    void si_calculated(GraphicsBranchItem*);

public:
    BranchCreationTask(PhyNode *n, const QFont &f, const QMap<PhyNode*, int> &a, int h);
    void run();
};

}//namespace;

#endif
