/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _CIRCULAR_ITEMS_H_
#define _CIRCULAR_ITEMS_H_

#include <QtGui/QGraphicsPathItem>
#include <QtGui/QFont>

#define Pi 3.1415926535897932384626433832795	

namespace GB2 {

class Annotation;
class AnnotationSettings;
class CircularAnnotationItem;
class CircularViewRenderArea;
class CircularAnnotationLabel;
class CircurlarAnnotationRegionItem;

/************************************************************************/
/* CircularAnnotationItem                                               */
/************************************************************************/

class CircularAnnotationItem : public QGraphicsItem {
friend class CircurlarAnnotationRegionItem;
friend class CircularAnnotationLabel;
public:	
	CircularAnnotationItem(Annotation* ann, CircurlarAnnotationRegionItem* region);
	CircularAnnotationItem(Annotation* ann, QList<CircurlarAnnotationRegionItem*>& regions);
	~CircularAnnotationItem();
	virtual void paint(QPainter *p,const QStyleOptionGraphicsItem *item,QWidget *widget);
	Annotation* getAnnotation() const;
	virtual QRectF boundingRect() const;
	virtual bool contains(const QPointF &point);
	int containsRegion(const QPointF &point);
	void setSelected(bool isSelected);
	const QList<CircurlarAnnotationRegionItem*>& getRegions();
protected:
	bool isSelected;
	QColor color;
	Annotation *annotation;
	QList<CircurlarAnnotationRegionItem*> regions;
	QRectF _boundingRect;
};

/************************************************************************/
/* CircurlarAnnotationRegionItem                                        */
/************************************************************************/
class CircurlarAnnotationRegionItem : public QGraphicsPathItem{
friend class CircularAnnotationItem;
public:
	CircurlarAnnotationRegionItem(const QPainterPath& path, bool isShort, int number );
	void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget /* = 0 */);
	CircularAnnotationLabel* getLabel() const;
	void setLabel(CircularAnnotationLabel* label);
protected:
	CircularAnnotationItem *parent;
	CircularAnnotationLabel *label;
	int number;
	bool isShort;
};

/************************************************************************/
/* CircularAnnotationLabel                                              */
/************************************************************************/
class CircularAnnotationLabel : public QGraphicsItem {
public:
	CircularAnnotationLabel(Annotation* ann, int _region, int sequenceLength, const QFont& font, CircularViewRenderArea* renderArea );
	virtual void paint(QPainter *p,const QStyleOptionGraphicsItem *item,QWidget *widget);
	Annotation* getAnnotation() const;
	virtual QRectF boundingRect() const;
	virtual bool contains(const QPointF &point);
	float getLabelY() const;
	int getRegion() const;

	bool operator<(const CircularAnnotationLabel& a);
	static void prepareLabels(QList<CircularAnnotationLabel*>& labelItems, const CircularViewRenderArea* renderArea);
protected:
	Annotation* annotation;
	float annotationAngle;
	float labelY;
	QFont labelFont;
	QRectF innerRect;
	QRectF outerRect;
	QPointF point1;
	QPointF point2;
	int region;
};

} // namespace GB2

#endif // _CIRCULAR_ITEMS_H_