/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _GB2_CIRCULAR_PAN_VIEW_H_
#define _GB2_CIRCULAR_PAN_VIEW_H_

#include <core_api/GObject.h>
#include <core_api/MainWindow.h>
#include <core_api/AppContext.h>
#include <core_api/DocumentModel.h>
#include <core_api/DNAAlphabet.h>
#include <core_api/ProjectModel.h>
#include <core_api/IOAdapter.h>
#include <core_api/Log.h>
#include <core_api/Task.h>

#include <util_ov_annotated_dna/ADVConstants.h>
#include <util_ov_annotated_dna/ADVSequenceObjectContext.h>
#include <util_ov_annotated_dna/AnnotatedDNAView.h>
#include <util_ov_annotated_dna/GSequenceLineViewAnnotated.h>

#include <QtGui/QFont>
#include <QtGui/QAction>
#include <QtGui/QScrollBar>
#include <QtGui/QScrollArea>


namespace GB2 {

class CircularViewRenderArea;
class GScrollBar;
class GObjectView;
class PVRowsManager;
class AnnotationTableObject;
class Annotation;
class CircularAnnotationItem;
class CircularAnnotationLabel;
class CircularViewSplitter;

class CircularView : public GSequenceLineViewAnnotated {
	friend class CircularViewSplitter;
	Q_OBJECT
public:
	CircularView(QWidget* p, ADVSequenceObjectContext* ctx);
	~CircularView();
	void pack();
	void mousePressEvent(QMouseEvent * e);
	void mouseMoveEvent(QMouseEvent * e);
	void mouseReleaseEvent(QMouseEvent* e);
	void resizeEvent(QResizeEvent* e);
	virtual QSize sizeHint() const;

	virtual QList<AnnotationSelectionData> selectAnnotationByCoord(const QPoint& coord) const;

protected slots: 
	virtual void sl_onAnnotationSelectionChanged(AnnotationSelection*, const QList<Annotation*>& added, const QList<Annotation*>& removed);
	virtual void sl_onDNASelectionChanged(LRegionsSelection* thiz, const QList<LRegion>& added, const QList<LRegion>& removed);
protected:
	QScrollArea* scrollArea;
	int lastMovePos;
	int currectSelectionLen;
	int lastMouseY;
};

class CircularViewRenderArea : public GSequenceLineViewAnnotatedRenderArea {
	friend class CircularView;
	friend class CircularAnnotationItem;
	friend class CircularAnnotationLabel;
	friend class CircurlarAnnotationRegionItem;
	Q_OBJECT
public:
	CircularViewRenderArea(CircularView* d);
protected:
	virtual void paintEvent(QPaintEvent *e);
	void resizeEvent(QResizeEvent *e);
	virtual void drawAll(QPaintDevice* pd);
	virtual LRegion getAnnotationYRange(Annotation* a, const LRegion& r, const AnnotationSettings* as) const;
	void buildAnnotationItem(DrawAnnotationPass pass, Annotation* a, bool selected = false, const AnnotationSettings* as = NULL);
	virtual void drawAnnotations(QPainter& p);
	void buildAnnotationLabel(const QFont& font, Annotation* a, const AnnotationSettings* as);
	void drawSequence(QPainter& p);
	void drawRuler(QPainter& p);
	void drawAnnotationsSelection(QPainter& p);
	void drawSequenceSelection(QPainter& p);
	CircularView* circularView;
private:
	static const int OUTER_ELLIPSE_SIZE = 512;
	static const int ELLIPSE_DELTA = 22;
	static const int INNER_ELLIPSE_SIZE = 480;
	static const int SEQUENCE_ELLIPSE_SIZE = INNER_ELLIPSE_SIZE - 20;
	static const int MIDDLE_ELLIPSE_SIZE = (INNER_ELLIPSE_SIZE + OUTER_ELLIPSE_SIZE) / 2;
	static const int RULER_ELLIPSE_SIZE = SEQUENCE_ELLIPSE_SIZE - 20;
	static const int ARROW_LENGTH = 32;
	static const int ARROW_HEIGHT_DELTA = 4;
	static const int LABEL_DEGREE_STEP = 20;
	static const int LABEL_DEGREES_COUNT = 90;
	static const int MAX_DISPLAYING_LABELS = 200;

	QList< QList <LRegion> > regionY;
	QMap< Annotation*, CircularAnnotationItem* > circItems;
	QMap< Annotation* , int > annotationYLevel;
	QList<CircularAnnotationLabel*> labelList;
	bool needResizeView;
	float scale;
};

}//namespace;

#endif
