/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "QScoreAdapter.h"
#include "qscore.h"

extern double QScore(MSA_QScore* _msaTest, MSA_QScore* _msaRef);

namespace GB2 {
	
	static void convertMAlignment2MSA(MSA_QScore& muscleMSA, const MAlignment& ma, bool fixAlpha){

		assert(ma.isNormalized());
		for (int i=0, n = ma.getNumSequences(); i<n; i++) {
			const MAlignmentItem& aseq = ma.alignedSeqs[i];

			char* seq  = new char[aseq.sequence.length()+1];
			memcpy(seq, aseq.sequence.data(), aseq.sequence.length());
			seq[aseq.sequence.length()] = '\0';

			char* name = new char[aseq.name.length() + 1];
			memcpy(name, aseq.name.toLocal8Bit().data(), aseq.name.length());
			name[aseq.name.length()] = '\0';

			muscleMSA.AppendSeq(seq, aseq.sequence.length(), name);
		}
		/*if (fixAlpha) {
			muscleMSA.FixAlpha();
		}*/
	}

	double QScore(const MAlignment& maTest, const MAlignment& maRef, TaskStateInfo& ti) {
		MSA_QScore msaTest, msaRef;
		try {
			convertMAlignment2MSA(msaTest, maTest, false);
			convertMAlignment2MSA(msaRef, maRef, false);
			return QScore(&msaTest, &msaRef);
		} catch(...) {
			ti.setError(QString("QScore throws exception: "));
			return 0;
		}

	}
} // namespace GB2 