/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "Muscle4Task.h"
#include "Muscle4Adapter.h"
#include <core_api/DocumentModel.h>
#include <core_api/Counter.h>


namespace GB2 {


Muscle4Task::Muscle4Task(const MAlignment& ma) 
:Task(tr("MUSCLE 4 alignment"), TaskFlags_FOSCOE), inputMA(ma)
{
    GCOUNTER( cvar, tvar, "Muscle4Task" );
}

void Muscle4Task::run() {
    assert(!hasErrors());
    Muscle4Adapter::align(inputMA, resultMA, stateInfo);

    if (!hasErrors() && !isCanceled()) {
        assert(resultMA.alphabet!=NULL);
        assert(resultMA.isNormalized());
    }
}


//////////////////////////////////////////////////////////////////////////
// MuscleGObjectTask

Muscle4GObjectTask::Muscle4GObjectTask(MAlignmentObject* _obj) 
: Task("", TaskFlags_NR_FOSCOE), obj(_obj), lock(NULL), muscleTask(NULL)
{
    QString aliName = obj->getDocument()->getName();
    setTaskName(tr("MUSCLE align '%1'").arg(aliName));
    setUseDescriptionFromSubtask(true);
    setVerboseLogMode(true);
}

Muscle4GObjectTask::~Muscle4GObjectTask() {
    assert(lock == NULL);
}

void Muscle4GObjectTask::prepare() {
    assert(true);
	if (obj.isNull()) {
        stateInfo.setError(tr("object_removed"));
        return;
    }
    if (obj->isStateLocked()) {
        stateInfo.setError(tr("object_is_state_locked"));
        return;
    }

    lock = new StateLock("muscle4_lock");
    obj->lockState(lock);
    muscleTask = new Muscle4Task(obj->getMAlignment());

    addSubTask(muscleTask);
}

Task::ReportResult Muscle4GObjectTask::report() {
    if (lock!=NULL) {
        obj->unlockState(lock);
        delete lock;
        lock = NULL;
    }
    propagateSubtaskError();
    if (hasErrors() || isCanceled()) {
        return ReportResult_Finished;
    }
    assert(!obj.isNull());
    if (obj->isStateLocked()) {
        stateInfo.setError(tr("object_is_state_locked"));
        return ReportResult_Finished;
    }
    obj->setMAlignment(muscleTask->resultMA); 

    return ReportResult_Finished;
}

} //namespace
